@@ -12,7 +12,7 @@ from loguru import logger
 from requests_futures.sessions import FuturesSession
 
 from refuel.types import DatasetEvent, Event, EventBatch
-from refuel.utils import DATE_FOURTEEN_DAYS_AGO, DATE_TOMORROW
+from refuel.utils import DATE_FOURTEEN_DAYS_AGO, DATE_TOMORROW, validate_event_schema
 
 
 class RefuelClient:
@@ -27,6 +27,7 @@ class RefuelClient:
     MAX_EVENTS_PER_BATCH = 100
     DEFAULT_QUEUE_NAME = "default"
     DEFAULT_DATASET_NAME = "default"
+    DEFAULT_VERSION_NAME = "default"
     DEFAULT_ORDERING = "recent"
     DEFAULT_MAX_QUERY_EVENTS = 1000000
     QUERY_STEP_SIZE = 100
@@ -99,6 +100,8 @@ class RefuelClient:
         y_true: Dict = None,
         metadata: Dict = None,
         dataset_name: str = DEFAULT_DATASET_NAME,
+        version: str = DEFAULT_VERSION_NAME,
+        validate_data: bool = False,
     ) -> pyfutures.Future:
         """
         Log individual events to the Refuel platform.
@@ -110,6 +113,7 @@ class RefuelClient:
             y_pred (dict): _description_. Defaults to None.
             y_true (dict, optional): _description_. Defaults to None.
             metadata (dict, optional): _description_. Defaults to None.
+            version (str, optional): model versioning. Defaults to "default".
 
         Returns:
             pyfutures.Future: _description_
@@ -125,12 +129,30 @@ class RefuelClient:
 
         client_timestamp = datetime.utcnow().strftime(self.DT_FORMAT)
         event = Event(
-            model_name, dataset_name, client_timestamp, x, y_pred, y_true, metadata
+            model_name,
+            dataset_name,
+            client_timestamp,
+            x,
+            y_pred,
+            y_true,
+            metadata,
+            version,
         )
+        if validate_data:
+            schema = self._init_context.get("schemas", {}).get(model_name, {})
+            if schema:
+                validate_event_schema(event, schema)
+            else:
+                logger.info(f"No model schema found. Skipping data validation.")
         return self._post(url=self._api_base_url + "/log", body=event.serialize())
 
     def log_batch(
-        self, model_name: str, events: List, dataset_name: str = DEFAULT_DATASET_NAME
+        self,
+        model_name: str,
+        events: List,
+        dataset_name: str = DEFAULT_DATASET_NAME,
+        version: str = "default",
+        validate_data: bool = False,
     ) -> List[pyfutures.Future]:
         """_summary_
 
@@ -138,6 +160,7 @@ class RefuelClient:
             model_name (str): _description_
             dataset_name (str): _description_
             events (List): _description_
+            version: (str): model version. Defaults to "default"
 
         Returns:
             List[pyfutures.Future]: _description_
@@ -155,7 +178,16 @@ class RefuelClient:
         chunks = self._chunkify_events(events)
         futures = []
         for chunk in chunks:
-            batch = EventBatch(model_name, dataset_name, client_timestamp, chunk)
+            batch = EventBatch(
+                model_name, dataset_name, client_timestamp, chunk, version
+            )
+            if validate_data:
+                schema = self._init_context.get("schemas", {}).get(model_name, {})
+                if schema:
+                    for e in batch.serialize():
+                        validate_event_schema(e, schema)
+                else:
+                    logger.info(f"No model schema found. Skipping data validation.")
             futures.append(
                 self._post(
                     url=self._api_base_url + "/log_batch", body=batch.serialize()
@@ -164,7 +196,12 @@ class RefuelClient:
         return futures
 
     def log_dataset(
-        self, model_name: str, dataset_name: str, cloud_uri: str, dataset_config: dict
+        self,
+        model_name: str,
+        dataset_name: str,
+        cloud_uri: str,
+        dataset_config: dict,
+        version: str = "default",
     ) -> pyfutures.Future:
         """Log dataset to the Refuel platform.
 
@@ -173,6 +210,7 @@ class RefuelClient:
             dataset_name (str): name of dataset
             cloud_uri (str): cloud uri or presigned url to dataset
             dataset_config (dict): configuration for the dataset
+            version (str): model version. Defaults to "default"
 
         Returns:
             pyfutures.Future:
@@ -189,9 +227,13 @@ class RefuelClient:
         client_timestamp = datetime.utcnow().strftime(self.DT_FORMAT)
 
         dataset_event = DatasetEvent(
-            model_name, dataset_name, client_timestamp, cloud_uri, dataset_config
+            model_name,
+            dataset_name,
+            client_timestamp,
+            cloud_uri,
+            dataset_config,
+            version,
         )
-
         return self._post(
             url=self._api_base_url + "/log_dataset", body=dataset_event.serialize()
         )
